%%%%%% 
% Fast approximated Soft Competitive Learning 
% for similarity and dissimilarity data
%
% In:
%     D - either 
%        + an N x d matrix of euclidean data, N cases 
%        + an N x N matrix of dissimilarities (make sure its valid - tested outside for efficiency)
%        + a vector with two matrices from the nystroem approximation [{N x p},{p x p}], p is the number of landmarks
%     dPercentOfLandmarks - percentage of landmarks with respect to N (e.g. 0.1 or 0.01)
%     iNrOfClusters       - expected number of clusters
%     (optional)
%     iCycles             - 100 (per default, may converge earlier)
%     fKernel    - a function pointer to a kernel function - if specified D is assumed to be Euclidean
% 
% Out:
%   
%
% Sample for kernel specification
%   if(0)
%       dGamma      = 0.5; % or what fits
%       fBaseKernel = @(X,Y)rbf_matlab(X,Y,dGamma);
%       bDiagConst  = true;
%	else    
%       bDiagConst  = false;
%       fBaseKernel = @(X,Y)X*Y';
%   end
%
% Evaluate (given Label - numeric N x 1 in L)
%   vWinners                      = argmin(distNeuronsToData);
%   dAcc = 0; vUnique = unique(vWinners); for(k=1:length(vUnique)),vField = vWinners==vUnique(k); vPosts(k) = mode(L(vField)); dAcc=dAcc+sum(L(vField) == vPosts(k));end;dAcc./length(L)
%
% (C) 2012 Frank-M. Schleif
% fschleif@techfak.uni-bielefeld.de
%
%%%%%%
function [cNeurons, distNeuronsToData, vAssignments, iCycle,dRunTime, dDualError] = fast_approx_scl(D,iNrOfClusters,dPercentOfLandmarks,iCycles,fKernel)

if(exist('fKernel','var')) %  convert data to dissimilarity
    [ny_Dis ~] = sim2dis_ny(fKernel,D,dPercentOfLandmarks);
elseif(~isvector(D))
    [ny_Dis ~] = dis2ny(D,dPercentOfLandmarks);
else
    ny_Dis = D;
end

if(~exist('iCycles','var')) % 
    iCycles = 100;
end
    
[cNeurons, distNeuronsToData, vAssignments, iCycle,dRunTime] = bng_relational_ny_scl( ny_Dis, iNrOfClusters , iCycles);
dDualError = 0;
if(nargout==6)
    U            = ny_Dis{1};
    T            = ny_Dis{2} * U';
    fKDistPtr    = @(X,Y)max(U(X,:) * T(:,Y),realmin);  % assumes true diss-matrix (maybe slightly incorr. due to nystroem approx)
    [dDualError] = determine_cost_function_dual( fKDistPtr, vAssignments);        
end

