function Dissimilarity = convert_similarity_matrix( Similarity, f )

%
% Converts Similarity Matrix to Dissimilarity Matrix
%
% Beforehand the Similarity Matrix will be made positive,
% normalized to [0,1]. Afterwards it is symmetrized.
%
% By default the conversion is done using d_ij = 1 - s_ij
%
% Define an optional conversion function:
%
%   f = @(M,kroot) 1 - (M.^(1/kroot));
%   g = @(M) f(M,5);
%   convert_similarity_matrix( Similarity, g )
%
% A.Hasenfuss (c) 2007
%
%  !!! Use with care and check the results !!!

% Default conversion function
if nargin < 2
  f = @(M) 1 - M;
end

% Make positive
minElem = min(min(Similarity));
if minElem < 0
  Similarity = Similarity - minElem + 10e-10;
end


% %A = SymSimilarity - diag(diag(SymSimilarity));
% A = SymSimilarity;
% B = reshape(A,1,1320*1320);
% hist(B,1000)

% Normalize to (0,1]
Similarity = Similarity / max(max(Similarity));

% Convert
Dissimilarity = f( Similarity );

% Symmetrize
[Dissimilarity] = symmetrize_matrix( Dissimilarity );

% Diagonal to zero
if(0)
Dissimilarity = Dissimilarity - 1;
end
% m = size(Dissimilarity, 1);
% for i = 1:m
%   Dissimilarity(i,i) = 0;
% end
