function [DualCostFuncVal, meanDist]  = determine_cost_function_dual( fKPtr, Assignment )

%
% Determines the value of the dual cost function for converged neurons
% It is also known as the intra-cluster distance
% In: 
%     fKPtr is a function pointer to a squared distance function
%   - Assignments 
%     vector with indices of the prototype the point is closest to
% Out:
%   - DualCostFuncVal - dual quantization error / intra-cluster distance 
%     (scales with the number of points
%
% A.Hasenfuss (c) 2008
% modified for CoreNG by F.-M. Schleif (c) 2012

Neurons = sort(unique(Assignment));

n = numel(Neurons);
meanDist = zeros(1,n);

for i = 1:n
  
  % Get receptive field
  nn  = Neurons(i);
  idx = (Assignment == nn);
  
  % Number of elements in receptive field
  vField = find(idx);
  num    = numel(vField);
 
  % shrink the number of calculation to a reasonable estimate
  vFieldSamples = randsample(num,min(max(1000,ceil(0.1*num)),num));
  vField        = vField(vFieldSamples);
  iFieldSize    = numel(vField);
  
  vSum          = zeros(iFieldSize-1,1);
  % diagonals are assumed to be zero
  for(l=1:iFieldSize-1)
      iRowIndex = vField(1);
      vField    = vField(2:end); % shrinks
      vSum(l)   = sum(fKPtr( iRowIndex, vField));      
  end
  dSumDistances = 2*sum(vSum); % add lower triangle
  
  % Mean of pairwise distances
  meanDist(i) = dSumDistances / iFieldSize; 
  
end

DualCostFuncVal = sum( meanDist ) / 4;
