function [Hitrate, corresp_Labels] = determine_hitrate( distNeuronsToData, Labels, NeuronLabels )
%
% Alexander Hasenfuss (c) 2005-2007
%
% Determines the hitrate of the labeled neurons on given labeled data by
% majority vote on the receptive fields
%
% distNeuronsToData   (n x m)-Distance matrix for m data points and n neurons
%

% Local variables
sum_blacksheep = 0;

% Needed values
[nn,mm] = size( distNeuronsToData );

if nn ~= numel( NeuronLabels )
    error('Dimension of NeuronLabels and DistDataToNeurons does not match!');
end;
    
% Return values
corresp_Labels = zeros(mm,1);

% Determine the corresponding neurons for all datapoints
[dummy, A] = min( distNeuronsToData, [], 1 );

% For each data point...
for i = 1:mm
    
    corresp_Labels(i) = NeuronLabels(A(i));
    
    if Labels(i) ~= corresp_Labels(i)
       sum_blacksheep = sum_blacksheep + 1; 
    end;
    
end;

Hitrate = 1 - (sum_blacksheep / mm);

