function [NeuronLabels, Classification, Assignment, Hitrate] = label_neurons( distNeuronsToData, Labels )
%
% Alexander Hasenfuss (c) 2005-2007
%
% Labels the neurons according to the majority-label of the data points within their
% receptive fields.
%
% It also stores the (fuzzy) classification for each neuron, the assignment of
% data to neurons, and the hitrate.
%
% Alternative to function "label_neurons_euclidean"
%
% Returns
% NeuronLabels ... (n x 1) Matrix 
% Classification ... {n x 1} Cell Array containing (k x 3) matrices with output of tabulate-command
% Assignment ... (m x 1) Matrix
%

% Local variables
sum_blacksheep = 0;

% Needed values
[num_neurons,num_data] = size( distNeuronsToData );

% Return values
NeuronLabels = zeros(num_neurons,1);
Classification = cell(num_neurons,1);

% Determine the corresponding neurons for all datapoints
[Assignment] = assign_datapoints_to_neurons( distNeuronsToData );

% For each neuron...
for i = 1:num_neurons,
    
    % ...determine the indexes of the corresponding datapoints
    idx = find( Assignment == i );
    
    if numel( idx ) ~= 0,
            
        % Get the labels of the corresponding datapoints (receptive field)
        corresp_labels = Labels( idx );
    
        % Count occurrence of label...
        tab = tabulate( corresp_labels );
    
        % ...and declare a winner label
        [dummy, max_idx] = max( tab(:,2) );
        winner_label = tab(max_idx,1);
        
        % Sum up black sheep
        sum_blacksheep = sum_blacksheep + (sum(tab(:,2)) - tab(max_idx,2));
    
        % Store represented labels and remove zero lines before
        Classification{i,1} = tab(find(tab(:,2) ~= 0),:);

    else
        winner_label = NaN;
    end;
    
    % Store winner label for this neuron
    NeuronLabels(i) = winner_label;

end;

Hitrate = 1 - (sum_blacksheep / num_data);
