function v = nmi(vOriginalLabel, vPostLabels)
% Nomalized mutual information
% Written by Mo Chen (mochen@ie.cuhk.edu.hk). March 2009.
% modification by Frank-M.Schleif
result  = vPostLabels(:);
label   = vOriginalLabel(:);
assert(numel(label) == numel(result));

label = label(:);
result = result(:);

n = length(label);

label_unique  = unique(label);
result_unique = unique(result);

% post result should not contain new labels, but may have missings
%assert(isempty(setdiff(result_unique,label_unique)));
c             = length(label_unique);

% frequency (distribution) original and result label
mTable    = tabulate(label);
Pl        = mTable(:,3)/100;
mTable    = tabulate(result);
[C,IA,IB] = intersect(mTable(:,1),label_unique);
Pr        = zeros(c,1); 
Pr(IB)    = mTable(IA,3)/100;

% entropy of Pr and Pl
Hl = -sum( Pl .* log2( Pl + eps ) );
Hr = -sum( Pr .* log2( Pr + eps ) );


% joint entropy of Pr and Pl
M = zeros(c);
for I = 1:length(result_unique)
	for J = 1:c
		M(I,J) = sum(result==result_unique(I)&label==label_unique(J));
	end;
end;
M = M / n;
Hlr = -sum( M(:) .* log2( M(:) + eps ) );

% mutual information
MI = Hl + Hr - Hlr;

% normalized mutual information
v = sqrt((MI/Hl)*(MI/Hr)) ;